function varargout = HTparaffin(in1,in2,in3)
% HTparaffin  Enthalpy versus temperature and its inverse, for phase change material
%
% HTparaffin:  H = HTparaffin(T)
%              T = HTparaffin(H,'inverse')
%              [rho,cp,k,lat,Tm] = HTparaffin('properties');
%              [dHdT,ifix,Tfix] = HTparaffin(T,'slope')
%
% Input:  T = temperature relative to melt temperature, i.e. T>0 for liquid
%             and T<0 for solid.  Units of T must be consistent with H
%         H = enthalpy of material
%
% Output: H = enthalpy of material when T is specified as input
%         T = temperature of material when H and `inverse' are
%             specified as input
%         rho = density of material
%         cps = specific heat of material in solid phase
%			 cpl = specific heat of material in liquid phase
%         ks  = thermal conductivity of material in solid phase
%			 kl  = thermal conductivity of material in liquid phase
%         lat = latent heat of material
%         Tm  = melting point of material
%         dHdT = slope of H(T) when T is specified as input
%         ifix = indices of nodes experiencing phase change
%         Tfix = phase change temperature

% --- Material properties from Table 2 in "Transient thermal management of
%     an avionics module using solid-liquid phase change materials",
%     D. Pal and Y.K. Joshi, ASME HTD-Vol.329, National Heat Transfer Conference,
%     Volume 7, 1996
cps = 2950;   %  specific heat of solid paraffin   (J/kg/K)
cpl = 2510;   %  specific heat of liquid paraffin   (J/kg/K)
rho = 818;    %  density of liquid or solid paraffin  (kg/m^3)
ks = 0.24;    %  thermal conductivity of solid paraffin,  (W/m/C)
kl = 0.24;    %  thermal conductivity of liquid paraffin,  (W/m/C)
L = 266e3;    %  latent heat  (J/kg)
Tm = 47;			%  melting temperature  (C)
kf = 204;	    %  thermal conductivity of aluminum heat sink (W/m/C)
rhof = 2707;  %  density of aluminum heat sink (kg/m^3)
cpf = 896;	  %  specific heat of aluminum heat sink ((J/kg/K)
%Lf = 1;		  %  very small latent heat of fin 	

if nargin==1
  varargout = {rho,cps,cpl,ks,kl,L,Tm,rhof,cpf,kf};
elseif nargin==2
  H = enthalpy(in1,in2,Tm,rho,cps,cpl,L,rhof,cpf);
  varargout = {H};
elseif nargin==3   %  in1 = H,  in2 = 'inverse'
  [dH,ifix,Tfix] = dHdTfun(in1,in2,Tm,rho,cps,cpl,L,ks,kl,rhof,cpf,kf);
  varargout = {dH,ifix,Tfix};
else
  error(sprintf('nargin = %d, but only one to three inputs are allowed',nargin));
end

% ========================================================================
function H = enthalpy(T,G,Tm,rho,cps,cpl,L,rhof,cpf)
% enthalpy  Evaluate enthalpy and optionally, dHdT, given temperature
%           relative to the melting point

global INDF

tol = 5*eps;       %  tolerance in deciding whether temperature is at melting
iL = find(G==1);              %  indices of nodes in liquid state
is = find(G==0);              %  nodes in solid state
im = find(G>0 & G<1);        %  nodes melting
if (length(iL)+length(is)+length(im))~=length(T)    %   simple checksum
   error('Number of liquid, solid and melting nodes not equal to total nodes')
end
H = zeros(size(T));   %  H should be input to avoid repeated memory allocations
H(iL) = rho*cpl*(T(iL)) + L;
H(is) = rho*cps*(T(is));
H(im) = rho*((1-G(im))*cps+G(im)*cpl).*(T(im)) + G(im)*L;
H(INDF) = rhof*cpf*(T(INDF));  %overwrites those solid portions within fin area

% ========================================================================
function [dH,ifix,Tfix] = dHdTfun(T,G,Tm,rho,cps,cpl,L,ks,kl,rhof,cpf,kf)
% enthalpy  Evaluate enthalpy and optionally, dHdT, given temperature
%           relative to the melting point

global INDF

im = find(G>0 & G<1);       %  nodes melting

dH = rho*(cpl-cps)*T.f + rho*L;   %  dH should be input to avoid repeated memory allocations
dH(INDF) = rhof*cpf*T.f(INDF);		 %  Overwrites those control volumes within fin	

T.rhocp = rho*((1-G)*cps + G*cpl);
T.rhocp(INDF) = rhof*cpf;				 %  Overwrites those control volumes within fin	
T.gamma =((1-G)*ks + G*kl);
T.gamma(INDF) = kf;						 %  Overwrites those control volumes within fin	

ifix = im;
Tfix = Tm;


