function PCmain(model,method,varargin)
% PCmain  Main program for transient 2D phase change simulation using CVFD method
%         and implicit Euler time integration.
%
% Synopsis: PCmain(model)
%	          PCmain(model,method)
%	          PCmain(model,method,opt1,opt2,...)
%
% Input:
%  	model = (string) name of mfile that defines the boundary conditions,
%	           source term, exact solution, and diffusion coefficient.
%	           Default:  model = 'model1'
%	  method = integer to specify solution method for the linearized system.
%	           method = 1  (default) for direct solution of A*x = b
%	           method = 2  preconditioned Conjugate Gradient with modified
%                        incomplete Cholesky preconditioner
%            method = 3  for Richardson iteration with frozen LU factoriztion
%                        which doesn't work (implementation erorr?)
%
%	  opt1,opt2,... =  optional arguments passed through to user-defined 'model'
%
% ----------------------------------------------------------------------------

%  Gerald Recktenwald and Jason Leland
%  Portland State University,
%  Mechanical Engineering Department
%  gerry@me.pdx.edu
%
%  Code development brach begun April 2000
%  Most recent update (to this comment statement) Sept 2002

global FPOUT                              %  Handle to output file 
if isempty(FPOUT), FPOUT = 1;  end        %  send to stdout as default

true = 1;  false = 0;    %  not defined in MATLAB 6.0 and earlier

if nargin<1   model='neumann1';              end  %  Use model problem #1 by default
if nargin<2 | isempty(method), method = 1;   end  %  Solve with direct method by default

% ---  Initialize Phi field by
%        o  creating the finite-volume grid
%        o  assigning boundary conditions, source term, diffusion coefficient
[ggrid,phi,tran] = feval(model,'initialize',varargin{:});

fprintf(FPOUT,'\n-------- Implicit Time Integration --------\n');
fprintf(FPOUT,'Linear equations solved with method = %d\n\n',method);

phi.timeMethod = 'implicit';
fivePtIterSolve(0);    %  intialize persistent variables 

% --- Use a wait bar if iteration results are not written to screen
if FPOUT~=1
	showWait = true;
	hwait = waitbar(0,'PC Heat sink simulation...');
	set(hwait,'Name',sprintf('Progress: max time = %6.2f',tran.tstop));
else
	showWait = false;
end

% --- Main loop for nonlinear iterations
tran.t = 0;
ktot = 0;      %  total number of nonlinear iterations in this simulation
kstep = 0;     %  total number of time steps
tic;           %  reset timer

fivePtImplicitStep;   %  purge persistent variables

for istep=1:tran.maxStep

  tran.istep = istep;
  tran.t = tran.t + tran.dt;
  tran.tsteps(istep) = tran.t;

  phiOld = phi;   %  a bit excessive.  No need to preseve old BC info, etc. (?)
  [phi,tran] = feval(model,'nextStep',ggrid,phi,tran);
  [phi,relres,kit] = fivePtImplicitStep(model,ggrid,phi,phiOld,tran,method);
  [phi,done] = feval(model,'stepReport',ggrid,phiOld,phi,tran,relres);
  ktot = ktot + kit;
  kstep = kstep + 1;

	if showWait
		waitbar(istep/tran.maxStep,hwait,sprintf('PC Heat Sink: t=%4.2f  r=%10.1e',tran.t,relres));
	end
  if done,  break;  end

end
tnet = toc;
if showWait, close(hwait); end

% --- Report the L2 norm of the error if exact solution is known
feval(model,'finalReport',ggrid,phiOld,phi,tran,tnet);
fprintf(FPOUT,'\n%12d time steps completed\n',kstep);
fprintf(FPOUT,'%12g nonlinear iterations completed\n',ktot);

% ===================================
function [phi,relres,kit] = fivePtImplicitStep(model,ggrid,phi,phiOld,tran,method)
% fivePtImplicitStep  One Implicit time step;  Nonlinear iterations on each step

persistent  A R r
if nargin<1
	%  reset persistent variables to force memory allocation for new ones
	A = [];	 M = [];  R = [];  r = [];
	return
end
true = 1;
false = 0;

alpha = 1;

M = [];
res = 0;
minit = 3;
itfac = 2;
kit = 0;            %  total number of iterations performed in this step
n = phi.nx*phi.ny;

% --- Main loop for nonlinear iterations
for iter=1:phi.maxit
   
  phi.iter = iter;

  % --- Update diffusion coefficients
  [phi,ifix,phiFix] = feval(model,'prepIter',ggrid,phi,tran);
  if iter==1
    [ae,aw,an,as] = fvcoeff(ggrid,phi);   % Compute CVFD coefficients
  end

  % --- Apply boundary conditions
  [ap,ae,aw,an,as,b] = fvbc(ggrid,phi,ae,aw,an,as);

  % ---  Add time coefficients
  ap = ap + phi.rhocp/tran.dt;
  b  = b  + phiOld.rhocp.*phiOld.f/tran.dt;
  
  % --- Under-relaxation
  b = b + ((1-alpha)/alpha)*ap.*phiOld.f;
  ap = ap/alpha;
	
	if method==1
    % --- Residual for this iteration
    if iter < minit
      relres = 1;   %  don't bother with residual calculation
    else
      res = fivePtResidualVec(phi.nx,phi.ny,ap,ae,aw,an,as,phi.f,b);
      relres = res/norm(b);
    end
		A = makeA(n,phi.nx,ae,aw,an,as,ap,A);
    phi.f = A\b;
    
  elseif method==2
		% --- Preconditioned conjugate gradient with modified incomplete Cholesky factorization
		A = makeA(n,phi.nx,ae,aw,an,as,ap,A);
		options.droptol = 0.1;
		options.michol = 1;
		tol = 5e-8;
		R = cholinc(A,options);
		[phi.f,flag2,relres] = pcg(A,b,tol,200,R',R);
    
  elseif method==3
		% --- Richardson iteration with frozen LU factorization:  doesn't work?
		if iter<3 | relres>10 | rem(iter,itfac) == 0
			A = makeA(n,phi.nx,ae,aw,an,as,ap,A);
			[L,U] = lu(A);
			phi.f = U\(L\b);
			relres = 1;
		else
			r = b - fivePtAxVec(phi.nx,phi.ny,ap,ae,aw,an,as,phi.f,r);
			phi.f = phi.f + 0.2*U\(L\r);
			relres = norm(r)/norm(b);
		end
	else
		error(sprintf('method = %d not allowed',method));
	end

  kit = kit + 1;
  if relres<phi.rtol
    converged = true;
    break;
  end

	% --- Update source terms and melt fraction field
	phi = feval(model,'postIter',ggrid,phi,tran,relres,ap,ifix);
	
end

% --- Compute relative residual if main program asks for no iterations
if phi.maxit==1
   relres = fivePtResidualVec(phi.nx,phi.ny,ap,ae,aw,an,as,phi.f,b)/norm(b);
end

% ===============================================
function A = makeA(n,nx,ae,aw,an,as,ap,A)
% makeA  Construct sparse coefficient matrix.  This should be more efficient

if isempty(A)
  A = sparse(1:n,1:n,ap,n,n) - sparse(1:n-1,2:n,ae(1:n-1),n,n) ...
                             - sparse(2:n,1:n-1,aw(2:n),n,n) ...
                             - sparse(1:n-nx,nx+1:n,an(1:n-nx),n,n) ...
                             - sparse(nx+1:n,1:n-nx,as(nx+1:n),n,n);
else
  A = spdiags(ap,0,A);                %  replace main diagonal of A
  A = spdiags([0; -ae(1:n-1)],1,A);   %  replace first superdiagonal of A
  A = spdiags([-aw(2:n); 0],-1,A);
  pad = zeros(nx,1);
  A = spdiags([pad; -an(1:n-nx)],nx,A);
  A = spdiags([-as(nx+1:n); pad],-nx,A);
end
