function v = cvfdProbe(xi,yi,grid,f)
% cvfdProbe  Interpolate CVFD field at a specified point in space
%
% Synopsis:  v = cvfdProbe(xi,yi,grid,f)
%
% Input:  xi,yi = coordinates of the point at which interpolant is evaluated
%         grid = data structure holding 2D CVFD grid information
%         f = data structure holding 2D CVFD field information
%
% Output:  v = value of f at (xi,yi)
%
% Note:  currently xi and yi must be scalars

if length(xi)>1, error('xi must be a scalar');  end
if length(yi)>1, error('yi must be a scalar');  end

%  Bilinear interpolation formula is from Numerical Recipes.
%  The point (labelled "o") is assumed to be contained inside
%  a box defined by four
%
%      (4) -------------- (3)
%       |                  |
%       |     o            |
%       |                  |
%       |                  |
%      (1) -------------- (2)
%
% Most of the logic of this function is required because the f data structure
% does not contain boundary values. 

nx = grid.nx;   ny = grid.ny;

% --- Find locations of known points in a rectangle that surrounds the
%     point at which interpolation is to be performed
if xi>grid.Lx | xi<0
  error('x location is not in the grid');
elseif xi<grid.x(1) & xi>=0
  i = 0;         xLeft = 0;  dx = grid.x(1);
elseif xi>grid.x(nx) & xi<=grid.Lx
  i = grid.nx;   xLeft = grid.x(nx);  dx = grid.x(nx+1)-xLeft;
else
  i = binsearch(grid.x,xi);   % search for i such that grid.x(i) <= x <= grid.x(i+1)
  xLeft = grid.x(i);
  dx = grid.x(i+1)-xLeft;
end

if yi>grid.Ly | yi<0
  error('y location in not in the grid');
elseif yi<grid.y(1) & yi>=0
  j=0;  yDown = 0;   dy = grid.y(1);
elseif yi>grid.y(ny) & yi<=grid.Ly
  j = grid.ny;   yDown = grid.y(ny);  dy = grid.y(ny+1)-yDown;
else
  j = binsearch(grid.y,yi);   % search for i such that grid.x(i) <= x <= grid.x(i+1)
  yDown = grid.y(j);
  dy = grid.y(j+1)-yDown;
end
[f1,f2,f3,f4] = findNeighbors(i,j,grid.nx,grid.ny,grid.x,grid.y,xi,yi,...
                              f.f,f.ebcval,f.wbcval,f.nbcval,f.sbcval);

% --- Bilinear interpolation formulas from Numerical Recipes
t = (xi-xLeft)/dx;
u = (yi-yDown)/dy;
v = (1-t)*(1-u)*f1 + t*(1-u)*f2 + t*u*f3 + (1-t)*u*f4;

% =============================================================
function [f1,f2,f3,f4] = findNeighbors(i,j,nx,ny,x,y,xi,yi,f,fe,fw,fn,fs)
% findNeighbors  find the values of f at the four neighbors on the box that
%                surrounds the point at which the interpolation is to be performed.
%
%      (4) -------------- (3)
%       |                  |
%       |     o            |
%       |                  |
%       |                  |
%      (1) -------------- (2)
%
% Synopsis: [f1,f2,f3,f4] = findNeighbors(i,j,nx,ny,x,y,xi,yi,f,fe,fw,fn,fs)
%
% Input:  i,j = indices of grid values such that
%                  x(i) <= xi < x(i+1)   and  y(j) <= yi <= y(j+1)
%         nx,ny = number of interior nodes in each direction
%         x,y   = vectors of locations for interior mesh points
%         xi,yi = (x,y) coordinates of points at which interpolant is to be evaluated
%         f     = vector of interior values in natural ordering
%         fe,fw = vectors of length ny defining field values on the east and west boundaries
%         fn,fs = vectors of length nx defining field values on the north and south boundaries
%
% Output: f1,f2,f3,f4 = values of f at the four points defining the corners of the box

% --- Determine the values of f based on the location of the point

if xi>=x(1) & xi<=x(nx) & yi>=y(1) & yi<=y(ny)
  
  % --- Probe is inside region of interior nodes
  n = i+(j-1)*nx;
  f1 = f(n);
  f2 = f(n+1);
  f3 = f(n+1+nx);
  f4 = f(n+nx);
  return;
  
elseif xi<x(1)  % --- Probe is between interior and west boundary

  if yi>=y(1) & yi<=y(ny) %  above bottom row and below top row
		n = 1+(j-1)*nx;
		f1 = fw(j);
		f2 = f(n);
		f3 = f(n+nx);
		f4 = fw(j+1);
		return;
     
  elseif yi<y(1)  %  Probe is in southwest corner
		f2 = fs(1);
		f3 = f(1);
		f4 = fw(1);
		f1 = planeInterp([x(1) x(1) 0],[0 y(1) y(1)],[f2 f3 f4],0,0);
    return;
    
  elseif yi>y(ny)  % Probe is in northwest corner
		f1 = f(1+(ny-1)*nx);
		f2 = fn(1);
		f4 = fw(ny);
    f3 = planeInterp([x(1) x(1) 0],[y(ny) y(ny+1) y(ny)],[f1 f2 f4],0,y(ny+1));
    return;

  else
    error('Probe position not found near west boundary');
  end
  
elseif xi>x(nx)  % --- Probe is between interior and east boundary

  if yi>=y(1) & yi<=y(ny) %  Above bottom row and below top row
		n = nx+(j-1)*nx;
    f1 = f(n);
    f2 = fe(j);
    f3 = fe(j+1);
    f4 = f(n+nx);
    return;

  elseif yi<y(1)   %  Probe is in southeast corner
		f2 = fe(1);
		f3 = f(nx);
		f4 = fs(nx);
    f1 = planeInterp([x(nx+1) x(nx) x(nx)],[y(1) y(1) 0],[f2 f3 f4],x(nx+1),0);
    return;

  elseif yi>y(ny)             %  node is in northeast corner
    f1 = fe(ny);
		f3 = fn(nx);
		f4 = f(nx*ny);
    f2 = planeInterp([x(nx+1) x(nx) x(nx)],[y(ny) y(ny+1) y(ny)],[f1 f3 f4],x(nx+1),y(ny+1));
    return;

  else
    error('Probe position not found near east boundary');
  end
  
elseif yi<y(1)  % --- Probe is between interior and south boundary
  
  if xi>=x(1) & xi<=x(nx)  %  Between west and east boundaries
    f1 = fs(i);
    f2 = fs(i+1);
    f3 = f(i+1);
    f4 = f(i);
    return;
  end

elseif yi>y(ny)  % --- Probe is between interior and north boundary
  
  if xi>=x(1) & xi<=x(nx)  %  Between west and east boundaries
    n = i+(j-1)*nx;
    f1 = f(n);
    f2 = f(n+1);
    f3 = fn(i+1);
    f4 = fn(i);
    return;
  end

else
  warning('interpolation rule not in place');
  [f1,f2,f3,f4] = fsafe(f,nx);  %  SAFE & INCORRECT
	return;
end

fprintf('\nNeighbors not found:\n\txi = %g, yi = %g\n',xi,yi);
fprintf('\tx(nx) = %g   x(nx+1) = %g\n',x(nx),x(nx+1));
fprintf('\ty(ny) = %g   y(ny+1) = %g\n',y(ny),y(ny+1));
error('Neighbors not found');


% ===============================
function v = planeInterp(x,y,z,xi,yi)
% planeExtrap  Interpolate a point lying in a plane defined by three points
%
% Input: x = three-element vector of x positions
%        y = three-element vector of y positions
%        z = three-element vector of values for the function z = f(x,y)
%        xi,yi = (x,y) coordinates of the point at which interpolant is to be evaluated
%
% Output:  v = value of planar function at (xi,yi)
a = plane([x(1) y(1) z(1)],[x(2) y(2) z(2)],[x(3) y(3) z(3)]);
v = a(1)*xi + a(2)*yi + a(3);

% ===============================
function a = plane(p1,p2,p3)
% plane   Find coefficients of equation for a plane in 3D.  Exercise 8-23
%
% Synopsis:  a = plane(p1,p2)
%            a = plane(p1,p2,p3)
%
% Input:  p1,p2,p3 = three points that define the plane;  Each p is a three-
%                    element vector giving the (x,y,z) coordinates of the
%                    point.  If p3 is not supplied, p3 = [0 0 0] is assumed.
%
% Output:  a = vector coefficients that define the plane satisfying
%                 a(1)*x + a(2)*y + a(3) = z

if nargin<3,  p3 = [0 0 0];   end   %  p3 is optional input

% --- assemble and solve equations
a = [p1(1)  p1(2)  1; p2(1)  p2(2)  1;  p3(1)  p3(2)  1]\[p1(3); p2(3); p3(3)];

