function [v,t] = cvfdProfile(direction,s,grid,f)
% cvfdProfile  Interpolate CVFD field along line of constant x or y
%
% Synopsis:  v = cvfdProfile(direction,s,grid,f)
%            [v,t] = cvfdProfile(direction,s,grid,f)
%
% Input:  direction = (string) `x' or 'y' to indicate coordinate
%                     that is held constant in obtaining the profile.
%                     direction = 'x' means that a vector of phi(y)
%                     values will be returned
%         s = value of the coordinate that is held constant
%         grid = data structure holding 2D CVFD grid information
%         f = data structure holding 2D CVFD field information
%
% Output:  v = vector of value of f(t) values at constant s
%
% Example:  v = cvfdProfile('x',0.5,grid,f)  returns a vector of
%               f(y) values at x=0.5
%
% Note:  currently s must be a scalar

switch lower(direction(1))    %  only look at first character in direction
  case 'x'
    if nargout==1
      v = yprofile(s,grid,f);
    elseif nargout==2
      [v,t] = yprofile(s,grid,f);
    else
      error('only one or two output arguments expected');
    end
  case 'y'
    if nargout==1
      v = xprofile(s,grid,f);
    elseif nargout==2
      [v,t] = xprofile(s,grid,f);
    else
      error('only one or two output arguments expected');
    end
  otherwise
    error(sprintf('profile along %s is not allowed',direction));
end

% ============================================================
function [phi,y] = yprofile(x,grid,f)
% yprofile  Interpolate between x values to get profile of phi(y)

i = find(x==grid.x);
if ~isempty(i)    %  x value is exactly on a grid point, skip the interpolation
  n = i + ( (1:grid.ny)-1 )*grid.nx;
  phi = [f.sbcval(i); f.f(n); f.nbcval(i)];
  if nargout==2,  y = [0; grid.y(:)];  end  
  return
end

% --- get x index for interpolation:  grid.x(i) <= x <= grid.x(i+1)
if x>grid.Lx | x<0,  error('x location is not in the grid');  end
i = binsearch(grid.x,x);   % search for i such that grid.x(i) <= x <= grid.x(i+1)
% --- Lagrange basis functions for the interpolation
s1 = (grid.x(i+1)-x)/(grid.x(i+1)-grid.x(i));
s2 = 1 - s1;
% --- indices of grid points along all y between two lines of constant x
%     where grid(x(i) <= x <= grid.x(i+1)
n = i + ( (1:grid.ny)-1 )*grid.nx;
ne = n + 1;
% --- vectorized interpolation
phi = s1*[f.sbcval(i); f.f(n); f.nbcval(i)] + s2*[f.sbcval(i+1); f.f(ne); f.nbcval(i+1)];
if nargout==2,  y = [0; grid.y(:)];  end  

% ============================================================
function [phi,x] = xprofile(y,grid,f)
% xprofile  Interpolate between y values to get profile of phi(x)

j = find(y==grid.y);
if ~isempty(j)    %  y value is exactly on a grid point, skip the interpolation
  n = (1:grid.nx) + ( j-1 )*grid.nx;
  phi = [f.wbcval(j); f.f(n); f.ebcval(j)];
  if nargout==2,  y = [0; grid.y(:)];  end  
  return
end

% --- get y index for interpolation:  grid.y(j) <= y <= grid.y(j+1)
if y>grid.Ly | y<0,  error('y location in not in the grid');  end
j = binsearch(grid.y,y);

% --- Lagrange basis functions for the interpolation
s1 = (grid.y(j+1)-y)/(grid.y(j+1)-grid.y(j));
s2 = 1 - s1;
% --- indices of grid points along all x between two lines of constant y
%     where grid(y(j) <= y <= grid.y(j+1)
n = (1:grid.nx) + (j-1)*grid.nx;
nn = n + grid.nx;
% --- vectorized interpolation
phi = s1*[f.wbcval(j); f.f(n); f.ebcval(j)] + s2*[f.wbcval(j+1); f.f(nn); f.ebcval(j+1)];
if nargout==2,  y = [0; grid.y(:)];  end  
