function cvfdSurfacePlot(grid,field,plotLabel)
% cvfdSurfacePlot  Generate a surface plot of the function u, where u is a
%                  one-dimensional array of only the interior points in the
%                  domain.  To use MATLAB's surface plotting routines we need
%                  the data stored in a two-dimensional array, uij,
%                  that includes the boundary values.  The grid arrays xi and yj
%                  are the locations of the grid *lines* i and j corresponding
%                  to the entries in uij.
%
% Synopsis:  cvfdSurfacePlot(grid,field)
%            cvfdSurfacePlot(grid,field,plotLabel)
%
% Input:  grid = 2D grid structure (see new2DgridStruct.m)
%         field = 2D field structure (see new2DfieldStruct.m)

%  Gerald Recktenwald, Portland State University, Mechanical Engineering Dept.
%  gerry@me.pdx.edu
%  June 1999, February 2000

nx = grid.nx;
ny = grid.ny;
nxp = nx+2;  nyp = ny+2;  Np = nxp*nyp;

%  pre-allocate memory for speed
uij = zeros(nxp,nyp);  xi  = zeros(nxp,1);  yi  = zeros(nyp,1);

%  Grid points used in the surface plot
xi(1) = 0.0;  xi(2:nxp-1) = grid.x(1:nx);  xi(nxp) = grid.xu(nx+1);
yj(1) = 0.0;  yj(2:nyp-1) = grid.y(1:ny);  yj(nyp) = grid.yv(ny+1);

% --- Copy interior nodes to the plotting matrix.  Using reshape would
%     be more elegant and efficient than the for loop.
for j=1:ny
  nstart = 1 + (j-1)*nx;
  uij(2:nxp-1,j+1) = field.f(nstart:nstart+(nx-1));    %  one x-direction row
end

%  Copy east and west boundary values
%   --- Transpose of uw and ue values are necessary because ...
%     uij is used in surface plot, and it has i (columns) corresponding to y
%     and j corresponding to y,
%     but matrix va
uij(1,  2:nyp-1) = field.wbcval';
uij(nxp,2:nyp-1) = field.ebcval';

%  Copy north and south boundary values
uij(2:nxp-1,1)   = field.sbcval';
uij(2:nxp-1,nyp) = field.nbcval';

%  Corner points are not defined precisely.  Use linear interpolation to
%  extend the north and south boundary values to the corners of the domain.
%  In other words extrapolate the corner values from the x-direction variation
%  along the north and south boundaries.  Note:  For a more general interpretation
%  of the corner value use two-dimensional interpolation.  That seemed excessive
%  in this demonstration program.
xrat = (xi(1)-xi(2))/(xi(3)-xi(2));               % southwest and northwest corners
uij(1,1)   = uij(2,1)   + xrat*(uij(3,1)   - uij(2,1));
uij(1,nyp) = uij(2,nyp) + xrat*(uij(3,nyp) - uij(2,nyp));

xrat = (xi(nxp)-xi(nxp-1))/(xi(nxp-2)-xi(nxp-1)); % southeast and northeast corners
uij(nxp,1)   = uij(nxp-1,1)   + xrat*(uij(nxp-2,1)   - uij(nxp-1,1));
uij(nxp,nyp) = uij(nxp-1,nyp) + xrat*(uij(nxp-2,nyp) - uij(nxp-1,nyp));

if length(xi) ~= length(yj)
  [XX,YY] = meshgrid(xi,yj);    %  meshc does not work with unequal x, y vectors ?
  meshc(XX,YY,uij');            %  transpose required;  should clean this up
                                %  with appropriate definition of uij, above
else
  meshc(xi(:),yj(:),uij);
end

if nargin == 3,   title(plotLabel);  end
axis('xy')