function [bal,Qvol,Qst,Qe,Qw,Qn,Qs] = eBalPhaseChange(grid,Hold,H,phiOld,phi,tran,verbose)
% eBalPhaseChange  Compute energy balance for transient 2D diffusion with source

%  NOTE:  The main program uses a switchyard to choose the energy balance
%         computation appropriate for the time integration method.



if nargin<5,  verbose=0;  end

if strncmp(lower(phi.timeMethod),'imp',3)
  [Qbal,Qgen,Qstore,Qeast,Qwest,Qnorth,Qsouth] = impBalance(grid,Hold,H,phiOld,phi,tran,verbose);
else
  error(sprintf('timeMethod = %s is not supported'))
end

if nargout>=1,  bal = Qbal;  end
if nargout>2
  Qvol = Qgen;  Qst = Qstore;
  Qe = Qeast;   Qw = Qwest;     Qn = Qnorth;  Qs = Qsouth;
end

return


% ============================================================================
function [Qbal,Qgen,Qstore,Qeast,Qwest,Qnorth,Qsouth] = impBalance(grid,Hold,H,phiOld,phi,tran,verbose)
% impBalance  Compute energy balance for transient 2D diffusion with source
%
% NOTE:   Storage, generation, and conduction fluxes are valid only
%         for Implicit time step scheme

% --- Volume integrals:
%       Qgen = heat generation during time step (positive for a gain)
%       Qstore = heat stored during time step (positive if increase in storage)

% --- SOURCE TERMS SHOULD BE LEFT IN to allow for heat generations
Qgen = sum( grid.dvol.*phi.src );
if ~isempty(phi.srcp)
  Qgen = Qgen + sum( grid.dvol.*phi.srcp.*phi.f );
end
Qstore = sum( grid.dvol.*(H-Hold)/tran.dt );  %  independent check on source terms

% --- Boundary integrals:
%       Qeast  = heat flow across east face in positive x-direction (out of domain)
%       Qwest  = heat flow across west face in positive x-direction (into of domain)
%       Qnorth = heat flow across north face in positive y-direction (out of domain)
%       Qsouth = heat flow across south face in positive y-direction (into of domain)
%
%  Implicit formulation requires that fluxes be evaluated at new time step

Qeast = 0;   Qwest = 0;  Qnorth = 0;   Qsouth = 0;
nx = grid.nx;    ny = grid.ny;
dxe = grid.xu(nx+1)-grid.x(nx);
dxw = grid.x(1);
for j=1:ny
  n = j*nx;
  qe = -phi.gamma(n)*(phi.ebcval(j)-phi.f(n))/dxe;
  Qeast = Qeast + qe*grid.dy(j);

  n = 1 + (j-1)*nx;
  qw = -phi.gamma(n)*(phi.f(n)-phi.wbcval(j))/dxw;
  Qwest = Qwest + qw*grid.dy(j);
end

dyn = grid.yv(ny+1)-grid.y(ny);
dys = grid.y(1);
for i=1:nx
  n = nx*ny - (nx-i);
  qn = -phi.gamma(n)*(phi.nbcval(i)-phi.f(n))/dyn;
  Qnorth = Qnorth + qn*grid.dx(i);

  qs = -phi.gamma(i)*(phi.f(i)-phi.sbcval(i))/dys;
  Qsouth = Qsouth + qs*grid.dx(i);
end

% --- Compute balance
Qbal = -Qstore + (Qgen + Qwest - Qeast + Qsouth - Qnorth);
if verbose,  printBalance(Qstore,Qgen,Qeast,Qwest,Qnorth,Qsouth,Qbal);  end


% ============================================================================
function printBalance(Qstore,Qgen,Qeast,Qwest,Qnorth,Qsouth,Qbal)
% printBalance  Utility to print terms in energy balance

global FPOUT
if isempty(FPOUT),  FPOUT=1;  end

fprintf(FPOUT,'\nEnergy balance:\n');
fprintf(FPOUT,' Qstore  = %12.3f\n',Qstore);
fprintf(FPOUT,' Qgen    = %12.3f\n',Qgen);
fprintf(FPOUT,' Qeast   = %12.3f\n',Qeast);
fprintf(FPOUT,' Qwest   = %12.3f\n',Qwest);
fprintf(FPOUT,' Qnorth  = %12.3f\n',Qnorth);
fprintf(FPOUT,' Qsouth  = %12.3f\n',Qsouth);
fprintf(FPOUT,' --------------------------\n');
fprintf(FPOUT,' Balance = %12.3f\n\n',Qbal);

return
