function y = fivePtAx(nx,ny,ap,ae,aw,an,as,x)
% fivePtAx  Computes y = Ax for the five-point difference scheme using
%           compass-point notation.  The coefficient array consists of
%     diagonalswhich are stored in the vectors ap, ae, aw, an, and as.
%     Nodes are numbered with natural ordering.  nx and ny are the
%     number of interior nodes (control volumes) in the x and y
%     directions, respectively.  Under these conditions an element of
%     the y vector is computed with
%
%	    y(i) = -as(i)*x(i-nx) - aw(i)*x(i-1) + ap(i)*x(i)
%              -an(i)*x(i+nx) - ae(i)*x(i+1)
%
% Synopsis:  y = fivePtAx(nx,ny,ap,ae,aw,an,as,x,y)
%
% Input:  nx,ny = number of unknowns on finite difference grid in x and y directions.
%         ap,ae,aw,an,as = diagonals of the coefficient matrix corresponding to the following
%                 stencil.  Mesh points are referred to with compass point notation where
%                 neighors of point "P" are "west" = (i-1,j), "north" = (i,j+1), etc.
%         x = input vector
%         y = output vector (is provided as input to allow reuse w/out memory allocation)
%
% Output:  y = A*x

% Note:  This routine could be optimized by using MATLAB vectorization expressions.
%        The point is to demonstrate the operations, not provide the most efficient
%        implementation.  The statements here can be directly translated to Fortran or C

n = nx*ny;
y = ap.*x;                                       %  diagonal term initializes y
y(1:n-1) = y(1:n-1) - ae(1:n-1).*x(2:n);         %  first superdiagonal
y(2:n)   = y(2:n)   - aw(2:n)  .*x(1:n-1);       %  first subdiagonal
nn = n - nx;
y(1:nn)   = y(1:nn)   - an(1:nn).*x((nx+1):n);     %  farthest superdiagonal
y((nx+1):n) = y((nx+1):n) - as((nx+1):n).*x(1:nn);     %  farthest subdiagonal
