function [u,M] = fivePtIterSolve(nx,ny,ap,ae,aw,an,as,b,method,M,verbose);
% fivePtIterSolve  Solve linearized system of equations arising from a 5 point stencil
%                  approximation to a PDE.  The coefficient matrix is stored as
%                  as five vectors using the compass point notation.  Persistent storage
%                  of A is 
%
% Synopsis:  u = fivePtIterSolve(0);
%            u = fivePtIterSolve(nx,ny,ap,ae,aw,an,as,b,method);
%            u = fivePtIterSolve(nx,ny,ap,ae,aw,an,as,b,method,M);
%            u = fivePtIterSolve(nx,ny,ap,ae,aw,an,as,b,method,M,verbose);
%            [u,M] = fivePtIterSolve(nx,ny,ap,ae,aw,an,as,b,method);
%            [u,M] = fivePtIterSolve(nx,ny,ap,ae,aw,an,as,b,method,M);
%            [u,M] = fivePtIterSolve(nx,ny,ap,ae,aw,an,as,b,method,M,verbose);
%
% Input:  with no inputs, or an input of 0, persistent variables are reset
%         and no calculations are performed.  Otherwise the following inputs
%         must be specified:
%
%    nx,ny = grid dimensions (unknowns only) in the x and y directions
%    ap    = diagonal coefficient
%    ae, aw, an, as, = off-diagonal coefficients corresponding to
%                      east, west, north and south neighbors
%    b      = right hand side vector
%    method = (optional) scalar or integer used to select solution method and
%             if applicable, a preconditioner.  The solution method is indicated
%             by the first element of method, and can be one of
%             of the following
%
%	   method(1)      solution method
%      ---------     -------------------
%          1         direct solution of A*u = b with sparse matrix A
%          2         plain conjugate gradient method
%          3         preconditioned conjugate gradient
%
%          8         preconditioned CG-squared
%          9         preconditioned BiCG-stab
%         10         preconditioned GMRES
%
%
%      preconditioners are selected via the second element of method
%
%	   method(2)      preconditioner
%      ---------     -------------------
%          1         diagonal of A
%          2         incomplete Cholesky
%          3         incomplete LU
%
%    M = (optional) preconditioning matrix.  If an iterative technique that uses
%        preconditioning is selected, and M is provided, then M is use as the
%        preconditioner.  Otherwise a new M is created as indicated by the
%        choice of 
%
%    verbose = (optional) flag to control message about total execution time
%              and sparsity of the A matrix.  Default: verbose = 0, no printing
%
% --------------------------------------------------------------

persistent A LL UU MM M1 M2   %   avoid, if possible, cost of allocating memory

selfTime = 0;  %  If true, use tic/toc to time the solve.  Turn off to allow outer loop timing

if nargin<=1   %  reset persistent variables
  A = [];  LL = [];  UU = [];  L = [];  M = [];  M1 = [];  M2 = [];
  return
end

if nargin<9,  method = 1;   end
if nargin<11, verbose = 0;  end

%    Set up the sparse matrix A with diagonals:  -as, -aw, ap, -ae, -an
%    This is needed for both the direct solution and the Krylov methods
n = nx*ny;
if isempty(A)
  A = sparse(1:n,1:n,ap,n,n) - sparse(1:n-1,2:n,ae(1:n-1),n,n) ...
                             - sparse(2:n,1:n-1,aw(2:n),n,n) ...
                             - sparse(1:n-nx,nx+1:n,an(1:n-nx),n,n) ...
                             - sparse(nx+1:n,1:n-nx,as(nx+1:n),n,n);
else
  A = spdiags(ap,0,A);               %  replace main diagonal of A
  A = spdiags([0; -ae(1:n-1)],1,A);   %  replace first superdiagonal of A
  A = spdiags([-aw(2:n); 0],-1,A);
  pad = zeros(nx,1);
  A = spdiags([pad; -an(1:n-nx)],nx,A);
  A = spdiags([-as(nx+1:n); pad],-nx,A);
end

% --- Direct solution
if method(1) == 1
   if selfTime, tic;  end
   u = A \ b;
   if selfTime, etime = toc; else, etime=0;  end
   if verbose
     if selfTime, fprintf('\n\t%f seconds elapsed time for direct solution of %dx%d system\n',etime,n,n);  end
     fprintf('\tFraction of nonzero elements in the A matrix is %g\n\n',nnz(A)/(n*n));
   end
   return;
end

% --- Prepare for iterative solution by Krylov methods

tol = 5.0e-12;   %  tolerance on ||r|| / ||b||
maxit = 3000;    %  maximum number of iterations

nn = nx*ny;   u = zeros(nn,1);  %  initial guess

% --- Construct a preconditioner as necessary
if length(method)==1
  pmethod = 0;
else
  pmethod = method(2);
end
switch pmethod
  case 1,   %  diagonal preconditioning
     if selfTime, tic;  end
     % L = ap;
     L = sparse(1:n,1:n,ap,n,n);   %  Mathworks routines need sparse matrix
     if selfTime, ptime = toc;  else, ptime=0;  end
     precFun = 'diagSolve';
  case 2,   %  incomplete Cholesky
     if selfTime, tic;  end
     if isempty(M)
       [mp,mew,mns] = fivePtLL(nx,ny,ap,aw,as);
       L = sparse(1:n,1:n,mp,n,n) - sparse(2:n,1:n-1,mew(2:n),n,n) ...
                                  - sparse(nx+1:n,1:n-nx,mns(nx+1:n),n,n);
     else
       L = M;  %  reuse an earlier
     end
     if selfTime, ptime = toc;  else, ptime=0;  end
     precFun = 'ICsolve';
  case 3,   %  incomplete LU
     if selfTime, tic;  end
     [LL,UU] = luinc(A);
     if selfTime, ptime = toc;  else, ptime=0;  end
     warning('Use of of ILU preconditioner is experimental');

  otherwise,
     precFun = 'noPreconditioner';
     ptime = 0;
end

% --- Solve
switch method(1)
  case 2,       %  plain conjugate gradient
   if selfTime, tic;  end
   [u,errEst,iter,flag] = cg(A,u,b,maxit,tol);
   if selfTime, stime = toc;  else, stime=0;  end
   methodName = 'plain CG';

  case 3,       %  preconditioned conjugate gradient
   if selfTime, tic;  end
   [u,flag,errEst,iter] = pcg(A,b,tol,maxit,L,L,u);         %  mathworks version of PCG
   % [u,errEst,iter,flag] = pcg(A,u,b,precFun,L,maxit,tol);
   if selfTime, stime = toc;  else, stime=0;  end
   methodName = ['preconditioned CG using ',precFun];

  case 8,   %  preconditioned conjugate gradient squared
   if selfTime, tic;  end
   [u,errEst,iter,flag] = cgs(A,u,b,precFun,L,maxit,tol);
   if selfTime, stime = toc;  else, stime=0;  end
   methodName = ['preconditioned CG-squared using ',precFun];

  case 9,   %  preconditioned BiCG-stab
   if selfTime,  tic;   end
   [u,errEst,iter,flag] = bicgstab(A,u,b,precFun,L,maxit,tol);
   if selfTime, stime = toc;  else, stime=0;  end
   methodName = ['BiCG-stab using ',precFun];

  case 10,  %  preconditioned GMRES

   %   form the Cholesky Factorization of "A"
   if selfTime, tic;  end
   restit = 64;   %  number of iterations between restarts
   if verbose
      fprintf('\n\tSolve with GMRES:  tol = %11.3e and restit = %d',tol,restit);
      fprintf('\tmay need adjustment\n');
   end
   [u,errEst,iter,flag] = gmres(A,u,b,'ICsolve',L,restit,maxit,tol);
   if selfTime, stime = toc;  else, stime=0;  end
   methodName = ['preconditioned GMRES using ',precFun];

  otherwise,
    error(sprintf('method = %s is not defined',method(1)));

end

if verbose
  fprintf('\n\tAfter %d iterations of %s\n',iter,methodName);
  fprintf('\t\tflag = %d, error estimate = %g\n',flag,errEst);
  if selfTime, fprintf('\t\telapsed time for solution = %f seconds\n\n',stime+ptime);  end
end

