function [x,xu] = fvUniMeshBlocks(nnx,xLens)
% fvUniMeshBlocks  Compute mesh variables for a mesh made up of blocks.
%                  Each block has a uniform mesh.
%
% Synposis:  [x,xu] = fvUniMeshBlocks(nx,xlen)
%
% Input:     nnx   = vector of numbers of control volumes in each block
%            xLens = vector block lengths
%
% Output:    x = vector of x-coordinates for nodes in center of control volumes
%            xu = vector of x-coordinates of left faces of control volumes
%
%                    x(i)
%               |---------->
%                      |         |       Vertical lines are the C.V. faces.
%                      |    o    |       "o" is location of node that is centered
%                      |         |       between the cell faces
%               |----->
%                 xu(i)
%
% Example:  A mesh of three blocks:
%
%                    xlens(1)        xlens(2)         xlens(3)
%                 |-----------|---------------------|---------|
%                     nnx(1)         nnx(2)           nnx(3)
%                     cells          cells            cells
%
%          The mesh inside each block is uniform, but the blocks themselves can
%          have different lengths, and different numbers of control volumes
%
%
%  Note:  This routine can be used to define grids in any coordinate direction, e.g.,
%             [x,xu] = fvUniMeshBlocks(nnx,xLens);
%             [y,yv] = fvUniMeshBlocks(nny,yLens);
%             [z,zw] = fvUniMeshBlocks(nnz,zLens);

if length(nnx)~=length(xLens),  error('nnx and xLens must have same number of elements');  end

% --- Loop over blocks.  For each block:
%       (1) store block grid data in temporary vectors xt and xut
%       (2) copy data into other temporary vectors that expand as blocks are added
%     After all blocks are constructed, copy data into final output variables.  The
%     final output variables will have data stored in contiguous locations in RAM
%     because they are created at once, not by being expanded as each block is added.
%     Final copy could be eliminated by preallocating x and xu, and then stuffing
%     xt and xut into appropriate parts of x and xu.  The method used here is easier
%     to read and a little wasteful of memory and data movement.
for i = 1:length(nnx)
  [xt,xut] = fvUniformMesh(nnx(i),xLens(i));
  if i==1
    xx = xt;  xxu = xut;        %  initialize
  else
    xx  = [xx(1:end-1);  xx(end)+xt ];    %  Remove last node of preceding block and add next block.  The
    xxu = [xxu(1:end-1); xxu(end)+xut ];  %  next block is offset by distance to end of preceding block
  end
end
x = xx;   xu = xxu;    % Copy to final output variables.