function [ap,ae,aw,an,as,b] = fvbc(grid,phi,ae,aw,an,as);
% fvbc  Apply boundary conditions to CVFD coefficients for diffusion a non-uniform two-dimensional mesh
%       with non-uniform diffusion coefficients.  Per unit volume formulation is used
%
%  Gerald Recktenwald, Portland State University, Mechanical Engineering Dept.
%  gerry@me.pdx.edu
%  May 1994, June 1999, May 2000, Sept 2002

%  Allocate memory and initilize coefficients
nx = grid.nx;  ny = grid.ny;  nn = nx*ny;
x = grid.x;    xu = grid.xu;  dx = grid.dx;
y = grid.y;    yv = grid.yv;  dy = grid.dy;
ap=zeros(nn,1);  b=phi.src;  gam = phi.gamma;

% --------------- Boundary Conditions -------------------------
%  Coefficients on the far east boundary;  No harmonic mean is used to
%  compute gamma_e because the east neighbor (boundary point) is on the
%  interface.  Compute the aeb value, use it to eliminate the boundary
%  value, then set the ae(np) to zero
delxe = xu(nx+1)-x(nx);
for j=1:ny
  np = nx + (j-1)*nx;               %  i=nx
  switch phi.ebctype(j)
    case 1  %  Dirichlet
      aeb = gam(np)/(delxe*dx(nx));
      b(np) = b(np) + aeb*phi.ebcval(j);
      ap(np) = ap(np) + aeb;
      ae(np) = 0.0;
    case 2  %  Neumann
      b(np) = b(np) + phi.ebcflux(j)/dx(nx);
      ae(np) = 0;
    case 3  %  Convection
      tmp = phi.ebchval(j)*phi.gamma(np)/(phi.ebchval(j)*delxe + phi.gamma(np));  %  h*k/(h*dyi + k)
      b(np) = b(np) + tmp*phi.ebctamb(j)/dx(nx);              % + tmp*Tinf/deltax
      ap(np) = ap(np) + tmp/dx(nx);
      ae(np) = 0;
    case 4  %  Symmetry
      ae(np) = 0;
    otherwise
      error(sprintf('BC type = %d not defined on east boundary',phi.ebctype(j)))
  end
end

%  Coefficients on the far north boundary.  No harmonic mean is used to
%  compute gamma_n because the north neighbor (boundary point) is on the
%  interface.  Follow analogous procedure to computing aeb, above.
delyn = yv(ny+1)-y(ny);
for i=1:nx
  np = i + (ny-1)*nx;               %  j=ny
  switch phi.nbctype(i)
    case 1  %  Dirichlet
      anb = gam(np)/(delyn*dy(ny));
      b(np) = b(np) + anb*phi.nbcval(i);
      ap(np) = ap(np) + anb;
      an(np) = 0.0;
    case 2  %  Neumann
      b(np) = b(np) + phi.nbcflux(i)/dy(ny);
      an(np) = 0;
    case 3  %  Convection
    	tmp = phi.nbchval(i)*phi.gamma(np)/(phi.nbchval(i)*delyn + phi.gamma(np));  %  h*k/(h*dyi + k)
      b(np) = b(np) + tmp*phi.nbctamb(i)/dy(ny);              % + tmp*Tinf/deltay
      ap(np) = ap(np) + tmp/dy(ny);
      an(np) = 0;
    case 4  %  Symmetry
      an(np) = 0;
    otherwise
      error(sprintf('BC type = %d not defined on north boundary',phi.nbctype(i)))
  end
end

%  Coefficients on the far west boundary.  Analogous to far east boundary
delxw = x(1);    %  xu(1) = x(0) = 0
for j=1:ny
  np = 1 + (j-1)*nx;                %  i=1
  switch phi.wbctype(j)
    case 1  %  Dirichlet
      awb = gam(np)/(delxw*dx(1));
      b(np) = b(np) + awb*phi.wbcval(j);
      ap(np) = ap(np) + awb;
      aw(np) = 0.0;
    case 2  %  Neumann
      b(np) = b(np) + phi.wbcflux(j)/dx(1);
      aw(np) = 0;
    case 3  %  Convection
      tmp = phi.wbchval(j)*phi.gamma(np)/(phi.wbchval(j)*delxw + phi.gamma(np));  %  h*k/(h*dyi + k)
      b(np) = b(np) + tmp*phi.wbctamb(j)/dx(1);               % + tmp*Tinf/deltax
      ap(np) = ap(np) + tmp/dx(1);
      aw(np) = 0;
    case 4  %  Symmetry
      aw(np) = 0;
    otherwise
      error(sprintf('BC type = %d not defined on west boundary',phi.wbctype(j)))
  end
end

%  Coefficients on the far south boundary.  Analogous to far north boundary
delys = y(1);    %  yv(1) = y(0) = 0
for i=1:nx
  np = i;                           %  j=1
  switch phi.sbctype(i)
    case 1  %  Dirichlet
      asb = gam(np)/(delys*dy(1));
      b(np) = b(np) + asb*phi.sbcval(i);
      ap(np) = ap(np) + asb;
      as(np) = 0.0;
    case 2  %  Neumann
      b(np) = b(np) + phi.sbcflux(i)/dy(1);
      as(np) = 0;
    case 3  %  Convection
      tmp = phi.sbchval(i)*phi.gamma(np)/(phi.sbchval(i)*delys + phi.gamma(np));
      b(np) = b(np) + tmp*phi.sbctamb(i)/dy(1);
      ap(np) = ap(np) + tmp/dy(1);
      as(np) = 0;
    case 4  %  Symmetry
      as(np) = 0;
    otherwise
      error(sprintf('BC type = %d not defined on south boundary',phi.sbctype(i)))
  end
end

%  Conservative scheme results in ap = sum of neighbors
ap = ap + ae + aw + an + as;        %  MATLAB sums over all n=1:nn
if ~isempty(phi.srcp)
  ap = ap - phi.srcp;     %  Conditional adding of this coefficient
end                       %  allows memory savings, phi.srcp = [] if
                          %  there is no source term coefficient

% printFivePtCoeff(nx,ny,ae,aw,an,as,ap,b)
