function g = new2DGridStruct(nx,ny,Lx,Ly,xu,yv)
% new2DGridStruct  Preallocate a 2D field structure
%
% Synopsis:  g = new2DGridStruct
%            g = new2DGridStruct(nx)
%            g = new2DGridStruct(nx,ny)
%            g = new2DGridStruct(nx,ny,Lx)
%            g = new2DGridStruct(nx,ny,Lx,Ly)
%            g = new2DGridStruct(nx,ny,Lx,Ly,xu)
%            g = new2DGridStruct(nx,ny,Lx,Ly,xu,yv)
%
% Input: nx,ny = number of control volumes in the x and y directions
%                Default:  nx = ny = 0;  (see below)
%        Lx,Ly = length of domain in x and y directions
%                Default:  Lx = Ly = 1;  (see below)
%        xu = vector of locations of control volume faces.  xu(i) = control
%             volume face on the negative side of node i
%        yv = vector of locations of control volume faces.  xu(i) = control
%             volume face on the negative side of node i
%
% Output:  g = struct containing complete description of the 2D CVFD grid
% 
%  g = new2DGridStruct  creates an empty struct with the following values
%         nx = ny = Lx = Ly = 0,  xu = yv = []
%
%  g = new2DGridStruct(nx)  creates a struct with
%         ny = nx;  Lx = Ly = 1;  x, xu are uniform grids,
%         y, yv are uniform grids

if nargin<1,  nx=0;   end
if nargin<2,  ny=nx;  end
if nargin<3,  Lx=1;   end
if nargin<4,  Ly=1;   end

x = [];  y = [];
if nargin<5,  [x,xu] = fvUniformGrid(nx,Lx);   end
if nargin<6,  [y,yv] = fvUniformGrid(ny,Ly);   end

% --- simple pre-emptive error checking
dx = diff(xu);
if any(dx==0)
  printFaceLocation(xu);
  error('Some dx<0');
end

dy = diff(yv);
if any(dy==0)
  printFaceLocation(yv);
  error('Some dy<0');
end

if isempty(x)
  x = zeros(1,nx+1);
  for i=1:nx,  x(i) = 0.5*(xu(i)+xu(i+1));  end
  x(nx+1) = Lx;
end

if isempty(y)
  y = zeros(ny+1,1);
  for j=1:ny,  y(j) = 0.5*(yv(j)+yv(j+1));  end
  y(ny+1) = Ly;
end

dv = zeros(nx*ny,1);   k = 0;
for j=1:ny
  for i=1:nx
    k = k + 1;
    dv(k) = dx(i)*dy(j);    %  could use outer product, no loops
  end
end
g = struct('nx',nx,'ny',ny,'Lx',Lx,'Ly',Ly,...
           'x',x,'xu',xu,'y',y,'yv',yv,...
           'dx',dx,'dy',dy,'dvol',dv);

% ======================================
function printFaceLocation(xu)

fprintf('\n\n   i     xu(i) or yv(j)    dx or dy\n');
fprintf('%4d   %9.5f\n',1,xu(1));
for i=2:length(xu)
  fprintf('%4d   %9.5f    %9.5f\n',i,xu(i),xu(i)-xu(i-1));
end
