function [x,xu] = uniformGrid(nx,xlen)
% uniformGrid  Defines finite-volume grid variables for a uniform mesh.
%
% Synposis:  [x,xu] = uniformGrid(nx,xlen)
%
% Input:     nx   = number of control volumes in the x direction
%            xlen = length of domain in the x direction
%
% Output:    x = vector of x-coordinates for nodes in center of control volumes
%            xu = vector of x-coordinates of left faces of control volumes
%
%                    x(i)
%             |---------->
%         
%                   |         |
%                   |    o    |
%                   |         |
%
%             |----->
%                 xu(i)
%
%  This routine can be used to define grids in any coordinate direction, e.g.,
%        [x,xu] = uniformGrid(nx,xLen);
%        [y,yv] = uniformGrid(ny,yLen);
%        [z,zw] = uniformGrid(nz,zLen);

x=zeros(nx+1,1);  xu=x;     %  pre-allocate memory and initialize

dx = xlen/nx;               %  Locate CV faces, xu
xu(1) = 0.0;
for i=2:nx+1
  xu(i) = xu(i-1) + dx;
end

for i=1:nx                  %  Center nodes between CV faces
  x(i) = 0.5 * ( xu(i+1) + xu(i) );
end
x(nx+1) = xu(nx+1);

