function varargout = halfFin(mode,varargin)
% halfFin  Transient heat transfer with phase change in a half-fin module
%          of a phase-change heat sink assembly.   Symmetry boundaries on far
%          east and west boundaries
%
% Synopsis:  Used with PCmain as follows
%
%    [outArgs] = halfFin(mode,inArgs)
%
%  where outArgs and inArgs are determined by the string variable, mode.
%
% Input:  mode = (required) string passed as first argument.  mode must be one of
%                'initialize' to create grid, set boundary values, etc.
%                'update' to update boundary values and other parameters that
%                         might change from one nonlinear iteration to the next
%                'report' to print final results of the analysis.
%         Lx, Ly = overall lengths of the domain in the x and y directions
%         Lf = length of fin above base (y-direction)
%         tb = thickness of base	(y-direction)
%         tf = fin thickness	(x-direction)
%         dxref = reference size for control volumes.  Number of grid lines
%                 in each subregion is chosen so that this size of control
%                 volume dimension is not exceeded anywhere in the domain.
%                 dxref controls both x and y direction grid spacing.
%
% Output: phi = 2D CVFD field structure with boundary values and source
%               term for this problem.  If mode = 'exact', the field values
%               returned in phi are the exact solution for the grid specified
%               by the input parameters.
%         grid = 2D CVFD grid structure.
%			    tran = Time step and transient data structure.
%
% Note:  Material properties are defined by the HofT and TofH functions
%        that evaluate the enthalpy as a function of temperature, and
%        temperature as a function of enthalpy, respectively.

if nargin<1   error(sprintf('Mode not set: (mode = %s)',mode));  end

switch lower(mode(1:4))   %   compare first four letters only
  case 'init'
    [ggrid,phi,tran] = initialize(varargin{:});
    varargout = {ggrid,phi,tran};
  case 'next'
    [phi,tran] = nextStep(varargin{:});
    varargout = {phi,tran};
  case 'upda'
    phi = update(varargin{:});
    varargout = {phi};
  case 'prep'
    [phi,ifix,Tfix] = prepIter(varargin{:});
    varargout = {phi,ifix,Tfix};
  case 'post'
    phi = postIter(varargin{:});
    varargout = {phi};
  case 'step'
    [phi,done] = stepReport(varargin{:});
    varargout = {phi,done};
  case 'fina'
  finalReport(varargin{:});
  otherwise
     error(sprintf('Mode = %s not valid'))
end

% =========================================================
% =========================================================
function [ggrid,phi,tran] = initialize(Lx,Ly,LB,bB,sw,dxref,dyref,dt,tstop)

global  FPOUT
global  G  Gold  dH  TSURF dFdT TolTm Finv
global  MELTING_CURVE_FCN INDF INDW
global  RHOCPS  RHOCPL  RHOCPF
global  HTAMB  TAMB  QBASE  TCRITICAL

fprintf(FPOUT,'%s started %s\n',mfilename,datestr(now));

% --- Set thermal parameters
MELTING_CURVE_FCN = 'HTparaffina';    %  Routine for specifying material properties
TCRITICAL = 80;                       %  Threshold for base temperature
HTAMB = 7220;                         %  Heat transfer coefficient on north face,  W/m^2/C 
TAMB = 80;				                    %  Ambient temperature on north face, C
QBASE = 16000;                        %  Heat flux on base, Watts/m^2
T0 = 25;                              %  Initial temperature

% --- Get material properties
[rho,cps,cpl,ks,kl,lat,Tm] = feval(MELTING_CURVE_FCN,'properties');
kf = 204;	    %  thermal conductivity of aluminum heat sink (W/m/C)
rhof = 2707;  %  density of aluminum heat sink (kg/m^3)
cpf = 896;	  %  specific heat of aluminum heat sink ((J/kg/K)

RHOCPS = rho*cps;     %  density times specific heat for solid PCM
RHOCPL = rho*cpl;     %  liquid PCM
RHOCPF = rhof*cpf;    %  aluminum fin
alphas = ks/rho/cps;  %  thermal diffusivity for solid
alphal = kl/rho/cpl;  %	 thermal diffusivity for liquid --- this may not be needed
alphaf = kf/rhof/cpf; %  thermal diffusivity for the heat sink

% --- Set intermediate quantities that allow grid to be sized
Lf = (LB-bB)*Ly;    %   length of fin from tip to top of the base:  total length is Lf + tb
tb = bB*Ly;         %   Thickness of base
tf = sw*Lx;         %   Thickness of fin
xgap = Lx - tf;
ygap = Ly - (Lf + tb);
if xgap<0 | ygap<0
  fprintf(FPOUT,'\nHere in initialization routine fin1:\n');
  fprintf(FPOUT,'\txgap = %f\n\tygap = %f\n',xgap,ygap);
  error('Fin can not be longer or wider than the domain.');
end

% --- Compute number of CVs in each x and y direction panel
nxf   = ceil(tf/dxref);
nxgap = ceil(xgap/dxref);
nyb   = ceil(tb/dyref);
nyf	  = ceil(Lf/dyref);
nygap = ceil(ygap/dyref);
nx = nxf + nxgap;
ny = nyb + nyf + nygap;

% --- Define location of control volume faces in the x-direction
xu = zeros(1,nx+1);
ifEnd = nxf;         	%  x-direction panel for fin thickness
dx = tf/nxf;
for i = 2:ifEnd+1,  xu(i) = xu(i-1) + dx;  end

ixgapBeg = ifEnd + 1;  	%  x-direction panel for gap right of fin
dx = xgap/nxgap;
for i=ixgapBeg+1:nx+1,  xu(i) = xu(i-1) + dx;  end

% --- define location of control volume faces in the y-direction
yv = zeros(ny+1,1);
jybEnd = nyb;
dy = tb/nyb;
for j = 2:jybEnd+1,  yv(j) = yv(j-1) + dy;  end

jfBeg = jybEnd + 1;
jfEnd = jfBeg + nyf - 1;
dy = Lf/nyf;
for j=jfBeg+1:jfEnd+1,  yv(j) = yv(j-1) + dy;  end

jygapBeg = jfEnd + 1;
dy = ygap/nygap;
for j=jygapBeg+1:ny+1,  yv(j) = yv(j-1) + dy;  end

% --- Print grid dimensions
fprintf(FPOUT,'\n\nParameters of Half-fin Model of Phase-Change Heat Sink:\n');
fprintf(FPOUT,'\tLx, Ly, nx, ny = %9.6f  %9.6f  %6d  %6d\n',Lx,Ly,nx,ny);
fprintf(FPOUT,'\tL/B, b/B, s/w  = %9.6f  %9.6f  %9.6f\n',(Lf+tb)/Ly,tb/Ly,tf/Lx);
fprintf(FPOUT,'\tdt, tstop      = %9.6f  %9.2f\n',dt,tstop);
fprintf(FPOUT,'\n\tLf, tb, tf     = %9.6f  %9.6f  %9.6f\n',Lf,tb,tf);
fprintf(FPOUT,'\txgap, ygap     = %9.6f  %9.6f  %9.6f\n',tb,Lf,ygap);
fprintf(FPOUT,'\n\trho,cps,ks     = %9.2f  %9.2f  %9.6f\n',rho,cps,ks);
fprintf(FPOUT,'\tcpl,kl         = %20.2f  %9.6f\n',cpl,kl);
fprintf(FPOUT,'\tlat,Tm         = %9g  %9.2f\n',lat,Tm);
fprintf(FPOUT,'\trhof,cpf,kf    = %9.2f  %9.2f  %9.2f\n',rhof,cpf,kf);

fprintf(FPOUT,'\nIndices of grid panels:\n');
fprintf(FPOUT,'\tifEnd ixgapBeg                = %3d  %3d  \n',ifEnd,ixgapBeg);
fprintf(FPOUT,'\tjybEnd  jfBeg  jfEnd jygapBeg = %3d  %3d  %3d  %3d\n',...
                 jybEnd,jfBeg,jfEnd,jygapBeg);
fprintf(FPOUT,'\n');

% --- Create the grid structure
ggrid = new2DGridStruct(nx,ny,Lx,Ly,xu,yv);

% --- diagnostic:  draw the grid and its features
makeGridPlot = 0;
if makeGridPlot
	plotGrid(ggrid)   
	hold on
	plot([gggrid.xu(ixgapBeg) ggrid.Lx],[ggrid.yv(jfBeg)  ggrid.yv(jfBeg)],'r-');
	plot([ggrid.xu(1) ggrid.xu(ifEnd+1)],[ggrid.yv(jygapBeg)  ggrid.yv(jygapBeg)],'r-');
	plot([ggrid.xu(ixgapBeg) ggrid.xu(ixgapBeg)],[ggrid.yv(jfBeg)  ggrid.yv(jygapBeg)],'r-');
	hold off
	title(sprintf('%d x %d grid for strangeCon problem',nx,ny));
	% --- Add problem parameters to the grid
	xtext = xu(nx+1) + 0.2*(ggrid.Lx);
	ytext = yv(ny+1) - 0.2*(ggrid.Ly);
	dy = 0.05*ggrid.Ly;
                        text(xtext,ytext,sprintf('Lx = %5.2f',ggrid.Lx));
	ytext = ytext - dy;  text(xtext,ytext,sprintf('Ly = %5.2f',ggrid.Ly));
	ytext = ytext - dy;  text(xtext,ytext,sprintf('Lf = %5.2f',Lf));
	ytext = ytext - dy;  text(xtext,ytext,sprintf('tb = %5.2f',tb));
	ytext = ytext - dy;  text(xtext,ytext,sprintf('tf = %5.2f',tf));
end

% --- Set up phi field:  Allocate memory and set convergence parameters
phi = new2DFieldStruct(nx,ny);
phi.maxit = 35;                  %  maximum number of nonlinear iterations per time step
phi.rtol = 5e-6;                 %  tolerance on relative residual for nonlinear iterations
phi.f = T0*ones(size(phi.f));		 %  Initial condition:  Uniform temperature at t=0

% --- Variable conductivity, source term, and rho*cp product
if T0<Tm
   G = zeros(size(phi.f));      				     %  initial liquid fraction
   phi.rhocp = rho*cps*ones(size(phi.f));    %  Uniform properties for solid IC
   phi.gamma = ks*ones(size(phi.gamma));
else
   G = ones(size(phi.f));       				     %  initial liquid fraction
   phi.rhocp = rho*cpl*ones(size(phi.f));    %  Uniform properties for solid IC
   phi.gamma = kl*ones(size(phi.gamma));
end

% --- Overwrite property values for regions occupied by the fin
for i=1:nx
  for j=1:jybEnd        %  Fin base
    n = i + (j-1)*nx;
    phi.gamma(n) = kf;
  	phi.rhocp(n) = rhof*cpf;  
  end
end
for i=1:ifEnd           % Vertical fin section
  for j=jfBeg:jfEnd
    n = i + (j-1)*nx;
    phi.gamma(n) = kf;
    phi.rhocp(n) = rhof*cpf;
  end
end

% --- Boundary conditions at initial time (t=0)
phi.nbctype = 3*ones(1,nx);                %  Convective boundary on the north
phi.nbchval = zeros(size(phi.nbcval));     %  True h and tamb values set in nextStep()
phi.nbctamb = zeros(size(phi.nbcval));
phi.sbctype = 2*ones(1,nx);		             %  Flux boundary on the south
phi.sbcflux = zeros(size(phi.sbctype));    %  True flux value is set in nextStep().
phi.wbctype = 4*ones(size(phi.wbctype));   %  Symmetry boundary on west
phi.ebctype = 4*ones(size(phi.ebctype));   %  Symmetry boundary on east
phi = update(ggrid,phi);                    %  Fix all auxillary boundary quantities

% --- Set up transient time step control
tran = newTimeStepControl(tstop,dt);   %  tstop and dt specified at input
% taus = (Lx-tf)^2/alphas;             %  taus and tauf not used?   
% tauf = (Lf+tb)^2/alphaf;

% --- Setting up misc needed variables

TolTm = 5e-2;
Gold = G;
INDF = find(phi.rhocp==(rhof*cpf));  %creates an index of nodes within the fin
INDW = find(phi.rhocp==(rho*cps));   %creates an index of nodes within PCM area.
if (length(INDF)+length(INDW)) ~= length(phi.rhocp)
  fprintf(FPOUT,'\n\n\tlength(INDF) = %d\n\tlength(INDW) = %d\n\tlength(phi.rhocp) = %d\n\n');
  error('INDF and INDW vectors do not include all internal CVs');
end

% --- allocate memory.  This also clears persistant variables from previous runs
dFdT = zeros(size(phi.f));
Finv = zeros(size(phi.f));


% =========================================================
% =========================================================
function phi = update(ggrid,phi);
% update  Adjust boundary values and temperature dependent properties at each nonlinear iteration

nx = phi.nx;  ny = phi.ny;

% --- Symmetry boundaries on east and west faces
for j=1:ny
  nw = 1 + (j-1)*nx;
  ne = nw + nx - 1;
  phi.wbcval(j) = phi.f(nw);
  phi.ebcval(j) = phi.f(ne);
end

% --- Flux boundary on south, convective boundary on north
for i=1:nx
   nn = i + (nx*ny-nx);
   phi.sbcval(i) = phi.f(i)+phi.sbcflux(i)*(ggrid.y(1))/(phi.gamma(i));
   
   kond = phi.gamma(nn)/(ggrid.yv(ny+1) - ggrid.y(ny));
   phi.nbcval(i) = (phi.nbchval(i)*phi.nbctamb(i)+kond*phi.f(nn))/(phi.nbchval(i)+kond);
   phi.nbcflux(i) = phi.nbchval(i)*(phi.nbctamb(i)-phi.nbcval(i));
end

% =========================================================
% =========================================================
function [phi,tran] = nextStep(ggrid,phi,tran);
% nextStep   Prepare for the next time step

global  FPOUT
global  G  Gold  dH
global  HTAMB  TAMB  QBASE
global  RHOCPS  RHOCPL  RHOCPF  INDF  INDW

Gold = G;  % set Gold = G for comparison after next run

% --- Update boundaries on North and South Faces
phi.sbcflux = QBASE*ones(size(phi.sbcval));   %  Constant flux/length on south boundary
phi.nbchval = HTAMB*ones(size(phi.nbcval));   %  Convective bc at north boundary
phi.nbctamb = TAMB*ones(size(phi.nbcval));

% --- Update material properties
phi.rhocp = G*RHOCPL + (1-G)*RHOCPS;
phi.rhocp(INDF) = RHOCPF;


% =========================================================
% =========================================================
function [phi,ifix,Tfix] = prepIter(ggrid,phi,tran)
% prepIter   Prepare for next nonlinear iteration:  update source terms

global FPOUT
global G  Gold  dH dFdT Tt TolTm Finv
global MELTING_CURVE_FCN INDF

dFdT(1:end) = 0;

Tt = phi.f;
[dH,ifix,Tfix] = feval(MELTING_CURVE_FCN,phi,G,'slope');

if ~isempty(ifix)
   dFdT(ifix) = 1/(2*TolTm);  %constrains temp to melting temp
end   

Finv = phi.f;
Finv(ifix) = Tfix;

phi.src  = -(dH/tran.dt).*(G - Gold - dFdT.*Finv);
phi.srcp = -(dH/tran.dt).*dFdT; 


% =========================================================
% =========================================================
function phi = postIter(ggrid,phi,tran,relres,ap,ifix)
% postIter   Computations at end of each nonlinear iteration

global FPOUT
global G  Gold  dH dFdT Tt Finv
global MELTING_CURVE_FCN INDF INDW TolTm

[dH,ifix,Tfix] = feval(MELTING_CURVE_FCN,phi,G,'slope');
[rho,cps,cpl,ks,kl,lat,Tm] = feval(MELTING_CURVE_FCN,'properties');

relax = 1;
if ~isempty(ifix)
   G = G + dFdT.*(phi.f-Finv);
else
   G = G + relax * ap.*(phi.f-Tt)./(dH/tran.dt+ ap./(dFdT + eps));
end

% --- Force change in g field for case where T>Tm but g<1
% +++ GWR added (begin), 5 Aug 2002
%    vectorize the following loopl:   NOT DONE CORRECTLY!!  DON'T USE THIS VECTORIZED CODE!
vectorCalc = 0;
if ~vectorCalc
  for i = 1:length(INDW)
    mc = INDW(i);
    if G(mc)<1 & phi.f(mc)>(Tm+1e-3)
      G(mc) = G(mc) + 1/(2*TolTm)*(phi.f(mc)-Finv(mc));  % forces the issue.
    end
  end
else
	imc = find( G(INDW)<1 & phi.f(INDW)>(Tm+1e-3) );
	if ~isempty(imc)
    G(imc) = G(imc) + 1/(2*TolTm)*(phi.f(imc)-Finv(imc));
	end
end
% +++ GWR added (end)

G(INDF)=0;
G(G<0) = 0;			%undershoot and overshoot correction
G(G>1) = 1;

% =========================================================
% =========================================================
function [phi,done] = stepReport(ggrid,phiold,phi,tran,relres)
% stepReport   Print results of this time step

global  FPOUT  BASEFNAME
global  G  Gold  dHdt Tt H Hold 
global  TPROBE  TCRITICAL INDF INDW MELTFRAC
global  MELTING_CURVE_FCN MELT_TIME OHTIME OHMELTFRAC
global 	EAMBIENT EBASE

persistent   XPROBE  YPROBE PRINT_INC

done = 0;
phi = update(ggrid,phi);   %  update boundary values

showSurf = 0;   %  flag to control whether surface plots are created during simulation

% --- print step summary at selected intervals
PRINT_INC = fix(tran.maxStep/20);
if PRINT_INC == 0,  PRINT_INC = 1;  end

% --- Reset global variables if this is the first time step
if tran.t==tran.dt
	XPROBE = [0 0.5*ggrid.Lx 0.5*ggrid.Lx];
	YPROBE = [0 0           0.5*ggrid.Ly];
  TPROBE = zeros(tran.maxStep,length(XPROBE));
  MELTFRAC = zeros(tran.maxStep,1);
  QnAve = zeros(tran.maxStep,1);
  EAMBIENT = 0;
  EBASE = 0;
  OHTIME = [];
  MELTTIME = [];
  fprintf(FPOUT,'\n\nTransient simulation of Phase Change Heat Sink: with phase change\n');
  fprintf(FPOUT,'\n istep   time    it    Tbase       Tp3     Ebal        relres        qn     melt frac\n');
end

% --- Compute energy balance
Hold = feval(MELTING_CURVE_FCN,phiold.f,Gold);  %   get enthalpies for energy balance
H = feval(MELTING_CURVE_FCN,phi.f,G);
Ebal = eBalPhaseChange(ggrid,Hold,H,phiold,phi,tran,0);

% --- Get average flux at north boundary
QnAve(tran.istep) = sum(phi.nbcflux.*ggrid.dx)/ggrid.Lx;

% --- Determine percentage of PCM melted
meltvol = sum(G(INDW).*ggrid.dvol(INDW));
PCMvol = sum(ggrid.dvol(INDW));
MELTFRAC(tran.istep) = meltvol/PCMvol;

% --- Get and save probe temperatures
%     First probe is the area-averaged base temperature
Tbsum = 0;
asum = 0;
for i=1:ggrid.nx
  Tbsum = Tbsum + phi.sbcval(i)*ggrid.dx(i);
  asum = asum + ggrid.dx(i);
end
if abs(asum-ggrid.Lx)>5*eps   %  error check: make sure we understand dimensions of domain
  error(sprintf('Base area computed incorrectly:  asum = %g   ggrid.Lx = %g',asum,ggrid.Lx));
end
Tbase = Tbsum/asum;
TPROBE(tran.istep,1) = Tbase;
for i=2:size(TPROBE,2)
	TPROBE(tran.istep,i) = cvfdProbe(XPROBE(i),YPROBE(i),ggrid,phi);
end
if abs(MELTFRAC(tran.istep)-1)<500*eps & Tbase>TCRITICAL,  done=1;  end

% --- Print results
if tran.t<=5*tran.dt | rem(tran.istep,PRINT_INC)==0
  fprintf(FPOUT,'%5d  %8.4f %3d  %8.2f  %8.2f  %10.2e  %10.2e  %8.2f  %8.4f\n',...
           tran.istep,tran.t,phi.iter,...
           Tbase,TPROBE(tran.istep,3),...
           Ebal,relres,QnAve(tran.istep),MELTFRAC(tran.istep));
end

% --- Record time when PCM is completely melted
if isempty(MELT_TIME)
  if abs(MELTFRAC(tran.istep)-1)<2*eps
    MELT_TIME = tran.t;    % saves the first time step where all is melted
    if showSurf
      fs = figure;
			cvfdSurfacePlot(ggrid,phi,...
          	sprintf('T field on a %d by %d grid at %5.1f seconds',phi.nx,phi.ny,tran.t));
      xlabel('x');   ylabel('y');  view(95,30)
      set(gca,'DataAspectRatio',[1 1 5000]);
      set(gcf,'Name',sprintf('T field at %-5.1f (s)',tran.t));
    end
	end
end

% -------- Find time overheating occurs and collect data -----------
if isempty(OHTIME)
  for i=1:ggrid.nx
    dAdt = ggrid.dx(i)*tran.dt;
    EBASE = EBASE + phi.sbcflux(i)*dAdt;
    EAMBIENT = EAMBIENT + phi.nbcflux(i)*dAdt;
  end
  if Tbase>TCRITICAL
    OHTIME = tran.t;                      %  Time when base exceeds threshold
    OHMELTFRAC = MELTFRAC(tran.istep);    %  Melt fraction when Tbase > TCRITICAL
    if showSurf
      fs = figure;
      cvfdSurfacePlot(ggrid,phi,...
          	sprintf('T field on a %d by %d grid at %5.1f seconds',phi.nx,phi.ny,tran.t));
      xlabel('x');   ylabel('y');  view(95,30)
      set(gca,'DataAspectRatio',[1 1 5000]);
      set(gcf,'Name',sprintf('T field at %-5.1f (s)',tran.t));
    end
  end
end

if (showSurf & rem(tran.istep,PRINT_INC)==0) | tran.istep==tran.maxStep
  fs = figure;
  cvfdSurfacePlot(ggrid,phi,...
      sprintf('T field on a %d by %d grid at %5.1f seconds',phi.nx,phi.ny,tran.t));
  xlabel('x');   ylabel('y');   view(95,30)
  set(gca,'DataAspectRatio',[1 1 5000]);
  set(gcf,'Name',sprintf('T field at %-5.1f (s)',tran.t));
end

% --- Specify time steps at which melt contour increments are to be drawn
contour_tvals = [50 100:100:1500 2000];      %  Time values at which contours are to be drawn
contour_nsteps = fix(contour_tvals/tran.dt + 1);  %  Time step numbers "  "  "  "  "

if any(tran.istep==contour_nsteps) | tran.istep==tran.maxStep
  fname = sprintf('%s_contour_t%d.mat',BASEFNAME,fix(tran.t));
  save(fname,'tran','ggrid','G','-mat');    %  write contour data to MAT file for post processing
end


% =========================================================
% =========================================================
function finalReport(ggrid,phiold,phi,tran,tnet)
% finalReport  Print results at end of computations 

global FPOUT BASEFNAME
global G  Gold  dHdT H Hold
global TPROBE  TCRITICAL  MELTFRAC
global MELTING_CURVE_FCN  INDF INDW
global OHTIME OHMELTFRAC  MELT_TIME  EAMBIENT  EBASE

% --- Update symmetry boundary conditions to be consistent with new interior solution
phi = update(ggrid,phi);

% --- Compute energy balance
[rho,cps,cpl,ks,kl,lat,Tm] = feval(MELTING_CURVE_FCN,'properties');
eBalPhaseChange(ggrid,Hold,H,phiold,phi,tran,1)

% --- Save scalar variables for comparisons with other runs
fname = sprintf('%s_scalars.mat',BASEFNAME);
tt = tran.tsteps;
save(fname,'tt','TPROBE','MELTFRAC','EAMBIENT','-mat');

% --- Plot probe temperatures versus time
fprobe = figure;
plot(tran.tsteps,TPROBE(:,1),'.',[min(tran.tsteps) tran.tstop],[TCRITICAL TCRITICAL],'r-');
xlabel('t (sec)');   ylabel('T_{p1}  (C)');
set(fprobe,'Name','Probe 1 temperature');

for i=2:size(TPROBE,2)
	fprobe = figure;
	plot(tran.tsteps,TPROBE(:,i),'.');   xlabel('t (sec)');   ylabel(sprintf('Probe %d temperature  (C)',i));
	set(fprobe,'Name',sprintf('Probe %d temperature',i));
end

% --- Plot domain-averaged melt fraction versus time
fmelt = figure;
plot(tran.tsteps,MELTFRAC,'.');   xlabel('t (sec)');   ylabel('f');
if ~isempty(OHTIME)
	hold on;
	plot([OHTIME OHTIME],[0 1],'r-');
  hold off;
end
set(fmelt,'Name','Melt fraction');

% --- Print summary results
fprintf(FPOUT,'\nPerformance of PC Heat sink:\n');
if ~isempty(MELT_TIME)
  fprintf(FPOUT,'\n\tWax was completely melted at t      = %10.2f seconds\n',MELT_TIME);
end
if ~isempty(OHTIME)
  fprintf(FPOUT,'\n\tThreshold temperature is %7.4f\n',TCRITICAL);
	fprintf(FPOUT,'\tAverage base temperature reached threshold at t = %7.4f seconds\n',OHTIME);
	fprintf(FPOUT,'\tAt that time:\n');
	fprintf(FPOUT,'\t\tMelt fraction = %7.5f\n',OHMELTFRAC);
	fprintf(FPOUT,'\t\tTotal heat transferred to base      = %10.2f J/m\n',EBASE);
	fprintf(FPOUT,'\t\tTotal heat transferred from ambient = %10.2f J/m\n\n',EAMBIENT); 
end
fprintf(FPOUT,'\n\tSimulation ended at t = %7.4f seconds\n',tran.t);
fprintf(FPOUT,'\tAt that time:\n');
fprintf(FPOUT,'\t\tFraction of PCM melted              = %10.4f\n',MELTFRAC(tran.istep));
fprintf(FPOUT,'\t\tAverage base temperature            = %10.2f C\n',TPROBE(tran.istep,1));
fprintf(FPOUT,'\t\tTotal heat transferred to base      = %10.2f J/m\n',EBASE);
fprintf(FPOUT,'\t\tTotal heat transferred from ambient = %10.2f J/m\n',EAMBIENT); 

% --- Print time stamp and final execution time 
fprintf(FPOUT,'\n%s ended %s\n',mfilename,datestr(now));
fprintf(FPOUT,'Net execution time = %g seconds\n',tnet);
