function [nx,ny] = meltContour(g,x,xu,y,yv,values,clabel,minPixels,fixLength)
% meltContour  Plot 2D melt front as determined by contours of liquid fraction.
%              Uses built-in contourc() and plot() functions
%
% Synopsis:  meltContour(g,x,xu,y,yv,values)
%            meltContour(g,x,xu,y,yv,values,clabel)
%            meltContour(g,x,xu,y,yv,values,clabel,fixLength)
%            [nx,ny] = meltContour(...)

if nargin<6,  error('Values of g, x, xu, y, yv and values must be supplied');  end
if nargin<7 | isempty(clabel),     clabel = [];      end
if nargin<8 | isempty(minPixels),  minPixels = 100;  end
if nargin<9 | isempty(fixLength),  fixLength = 1;    end

% --- Creation of contour plot consists of these steps
%  1.  Map the g field, which is stored as a column vector of interior values,
%      into a matrix that contains boundary values
%  2.  Interpolate the matrix of values on the non-uniform grid to a matrix
%      of values on a fine, uniform grid.  The fine matrix has a number of rows
%      and columns that corresponds to the true aspect ratio of the domain
%  3.  Draw the contours.

% --- Create grid vectors xi and yj that contain locations of interior and boundary nodes
nx = length(x) - 1;
ny = length(y) - 1;
xx = x(:);   yy = y(:);            %  temporary column vectors
xi = [0;  xx(1:nx);  xu(nx+1)];    %  coordinates of nodes including boundary nodes
yj = [0;  yy(1:ny);  yv(ny+1)];

G = makeGmatrix(g,nx,ny);
[X,Y,Z] = fineScaleImage(xi,yj,G,minPixels,fixLength);
doContours(Z,values,'-',clabel)

% --- Make surface plot to verify contour plot
debug = 0;
if debug
	figure;
	surf(X,Y,Z);
	xlabel('X axis')
	axis('equal');
	axis('tight');
end

if nargout>0
	nx = size(Z,2);   %  nx and ny allow scaling of physical data to units
	ny = size(Z,1);   %  used in the contour plot
end

% ===================================================================================
function GG = makeGmatrix(g,nx,ny);  %  ,xu,y,yv,values)

% --- Convert g field to a matrix format that includes boundary values
%       NOTE:  The GG matrix is first constructed so that 
nxp = nx+2;  nyp = ny+2;  %  number of nodes on the plot grid (includes boundary nodes)
GG = zeros(nyp,nxp);
GG(2:nyp-1,2:nxp-1) = flipud(reshape(g,nx,ny)');   %  stores values of interior nodes

% --- Boundary values of g are obtained from symmetry boundary conditions
%     This involves copying the rows or columns of values adjacent to the boundary
%     to the rows or columns on the boundary
ixrow = 2:(nxp-1);                %  column indices for interior nodes along the x-direction
GG(1,ixrow) = GG(2,ixrow);        %  south boundary
GG(nyp,ixrow) = GG(nyp-1,ixrow);  %  north boundary
jyrow = 2:(nyp-1);                %  row indices for interior nodes along the y-direction
GG(jyrow,1) = GG(jyrow,2);        %  west boundary
GG(jyrow,nxp) = GG(jyrow,nxp-1);  %  east boundary

% --- Fix corners
GG(1,1) = GG(2,2);  GG(1,nxp) = GG(1,nxp-1);
GG(nyp,1) = GG(nyp,2);  GG(end,end) = GG(end-1,end-1);

GG = flipud(GG);

% ===================================================================================
function [X,Y,Z] = fineScaleImage(xi,yj,G,minPixels,fixLength)

if nargin<4, minPixels=100;  end

xlen = max(xi)-min(xi);
ylen = max(yj)-min(yj);
if nargin<5 | isempty(fixLength)
	fixLength = min( [xlen,ylen] );
end

% --- Find scale of uniform mesh that preserves aspect ratio of original nonuniform mesh
scaleFactor = fix( minPixels/fixLength + 0.5);
nnx = fix( xlen*scaleFactor + 0.5 );
nny = fix( ylen*scaleFactor + 0.5 );
xx = linspace(min(xi),max(xi),nnx);
yy = linspace(min(yj),max(yj),nny);

[X,Y] = meshgrid(xx,yy);
Z = griddata(xi',yj,G,X,Y);


% =====================================================================================
function doContours(Z,values,symb,clabel)

if nargin<3,  symb = '.';       end
if nargin<4 | isempty(clabel),  clabel=[];   end
debug = 1;

% --- Let MATLAB find the points along a contour
if length(values)==1
  C = contourc(Z,[values values]);     %  Get contour matrix for one contour level
else
  C = contourc(Z,values);              %  Get contour matrix for list of contour levels
end
[mc,nc] = size(C);
if debug
  fprintf('\tC has %d rows and %d columns\n',mc,nc);
  fprintf('\tmin(Z), max(Z) = %g %g    values = ',min(min(Z)),max(max(Z)));
  fprintf('  %g',values);
  fprintf('\n');
end
if nc==0   %  no contours found
  if debug,  fprintf('\n');  end
  return;
end

% --- Loop over segments of contours
ibeg = 2;                %  row index of first data pair for current contour
holdstate = ishold;
while ibeg<nc
  % --- Get descriptive data about this group of points on this contour
	lev = C(1,ibeg-1);     %  level of the current contour
	np = C(2,ibeg-1);      %  number of points in the (x,y) data set that defines the contour
  iend = ibeg+np-1;      %  row index of first data pair in the contour
  if debug, fprintf('\tlev,np,iend = %6d %6d %6d\n',lev,np,iend);  end
  
  % --- Extract the contour points for this segment from C, and plot the segment.
	xc = C(1,ibeg:iend);
	yc = C(2,ibeg:iend);
  if ibeg>2 & ~holdstate,  hold on;  end
  plot(xc,yc,symb);
  ibeg = iend + 2;     %  prepare for next contour segment

  % --- Add contour labels.  Heuristic rules: put it at top and bottom of contour, and at far right
  itt = find(yc==max(yc));  itt = itt(1);  %  index of point for text label:  x(it), y(it)
  itb = find(yc==min(yc));  itb = itb(1);
  itr = find(xc==max(xc));  itr = itr(1);
  if isempty(clabel)
    text(xc(itt),yc(itt),sprintf('v = %-6.2f',lev));
    text(xc(itb),yc(itb),sprintf('v = %-6.2f',lev));
    text(xc(itr),yc(itr),sprintf('v = %-6.2f',lev));
  else
    text(xc(itt),yc(itt),clabel);
    text(xc(itb),yc(itb),clabel);
    text(xc(itr),yc(itr),clabel);
  end
end

if debug,  fprintf('\n');  end

% --- Draw a border around the extend of the domain.  This is a hack to
%     allow the sequence
%          axis('equal')
%          axis('tight')
%     to set the proper aspect ratio of the axis and to trim the plot
%     to its correct boundaries

[m,n] = size(Z);
plot([0 n n 0 0],[0 0 m m 0],'k-');   %  yes, m and n are swapped
axis('equal');
axis('tight');

if ~holdstate, hold off;  end

